@@ -18,31 +18,31 @@ test_that("inverse of base 'which' function", {
   expect_equal(invwhich(c(1, 2), len = 3), c(TRUE, TRUE, FALSE))
 })
 
-test_that("asCharacterNoWarn", {
-  expect_error(asCharacterNoWarn())
-  expect_equal(asCharacterNoWarn(1), "1")
-  expect_equal(asCharacterNoWarn(1L), "1")
-  expect_equal(asCharacterNoWarn(1.1), "1.1")
-  expect_equal(asCharacterNoWarn(c(1.2, 34.555)), c("1.2", "34.555"))
+test_that("as_char_no_warn", {
+  expect_error(as_char_no_warn())
+  expect_equal(as_char_no_warn(1), "1")
+  expect_equal(as_char_no_warn(1L), "1")
+  expect_equal(as_char_no_warn(1.1), "1.1")
+  expect_equal(as_char_no_warn(c(1.2, 34.555)), c("1.2", "34.555"))
 
-  expect_equal(asCharacterNoWarn(NA), NA_character_)
-  expect_equal(asCharacterNoWarn(c(1.2, NA)), c("1.2", NA_character_))
-  expect_equal(asCharacterNoWarn(c(NA, 1.2)), c(NA_character_, "1.2"))
+  expect_equal(as_char_no_warn(NA), NA_character_)
+  expect_equal(as_char_no_warn(c(1.2, NA)), c("1.2", NA_character_))
+  expect_equal(as_char_no_warn(c(NA, 1.2)), c(NA_character_, "1.2"))
 })
 
-test_that("asCharacterNoWarn factor", {
-  expect_equal(asCharacterNoWarn(factor(1)), "1")
-  expect_equal(asCharacterNoWarn(factor(1L)), "1")
-  expect_equal(asCharacterNoWarn(factor(1.1)), "1.1")
-  expect_equal(asCharacterNoWarn(factor(c(1.2, 34.555))), c("1.2", "34.555"))
+test_that("as_char_no_warn factor", {
+  expect_equal(as_char_no_warn(factor(1)), "1")
+  expect_equal(as_char_no_warn(factor(1L)), "1")
+  expect_equal(as_char_no_warn(factor(1.1)), "1.1")
+  expect_equal(as_char_no_warn(factor(c(1.2, 34.555))), c("1.2", "34.555"))
 
-  expect_equal(asCharacterNoWarn(factor(NA)), NA_character_)
-  expect_equal(asCharacterNoWarn(factor(c(1.2, NA))), c("1.2", NA_character_))
-  expect_equal(asCharacterNoWarn(factor(c(NA, 1.2))), c(NA_character_, "1.2"))
-  expect_equal(asCharacterNoWarn(factor(NA, exclude = FALSE)), NA_character_)
-  expect_equal(asCharacterNoWarn(factor(c(1.2, NA), exclude = FALSE)),
+  expect_equal(as_char_no_warn(factor(NA)), NA_character_)
+  expect_equal(as_char_no_warn(factor(c(1.2, NA))), c("1.2", NA_character_))
+  expect_equal(as_char_no_warn(factor(c(NA, 1.2))), c(NA_character_, "1.2"))
+  expect_equal(as_char_no_warn(factor(NA, exclude = FALSE)), NA_character_)
+  expect_equal(as_char_no_warn(factor(c(1.2, NA), exclude = FALSE)),
                c("1.2", NA_character_))
-  expect_equal(asCharacterNoWarn(factor(c(NA, 1.2), exclude = FALSE)),
+  expect_equal(as_char_no_warn(factor(c(NA, 1.2), exclude = FALSE)),
                c(NA_character_, "1.2"))
 })
 
@@ -403,7 +403,7 @@ test_that("areIntegers", {
   expect_that(areIntegers(c(1, 2)), testthat::equals(c(TRUE, TRUE)))
   expect_true(areIntegers(-1))
   expect_true(all(areIntegers(zeroes)))
-  expect_that(all(areIntegers(random_test_integers())), is_true())
+  expect_true(all(areIntegers(random_test_integers())))
   expect_true(areIntegers(1.00000000005))  # inside default tolerance
   expect_that(areIntegers(NA_integer_, na.ignore = T),
               testthat::equals(NA_integer_))
@@ -411,16 +411,12 @@ test_that("areIntegers", {
               testthat::equals(NA_integer_))
   expect_that(areIntegers(NA_character_, na.ignore = T),
               testthat::equals(NA_integer_))
-  expect_that(areIntegers(NA_integer_, na.ignore = F),
-              testthat::is_false())
-  expect_that(areIntegers(NA_real_, na.ignore = F),
-              testthat::is_false())
-  expect_that(areIntegers(NA_character_, na.ignore = F),
-              testthat::is_false())
+  expect_false(areIntegers(NA_integer_, na.ignore = F))
+  expect_false(areIntegers(NA_real_, na.ignore = F))
+  expect_false(areIntegers(NA_character_, na.ignore = F))
   expect_true(areIntegers(1.005, tol = 0.01))
-
   # multi value:
-  expect_that(areIntegers(c(1, 0, -1)), testthat::equals(c(T, T, T)))
+  expect_equal(areIntegers(c(1, 0, -1)), c(T, T, T))
   expect_equal(areIntegers(zeroes), rep(TRUE, times = length(zeroes)))
   expect_equal(areIntegers(random_test_integers()),
                rep(TRUE, times = length(random_test_integers())))
